#ifndef bqtRockEditDEFsHH
#define bqtRockEditDEFsHH

#define USE_ROM_EXPANSION 1

/*
  C-I-B-F-E-G CIBFEG
  
  TODO:
    Shutter definitions (doable)
  TODO:
    Buzzblock definitions (difficult)
  TODO:
    Transporter definitions (difficult)
  
  UNHANDLED?
      
    Table C2D4: (OK)stage restart points A
    Table C2E0: (OK)stage restart points B
    Table C2EB: (OK)level begin points
    Table C2F7: (OK)restart point A screen (?)
    Table C303: (OK)restart point B screen (?)
    
    Table 059380: (OK)keys that are automatically held when arriving in boss's room
                 0=left, $20=up, $10=down
   
    Table 059F0D: (OK)palettes for after-doors
                 16 bytes each stage
    
    Table 0593A0: (OK)order positions at start (0,0,0,0,0,0, 14,7,9,9,22,7)
    Table 0593AC: (OK)order positions at checkp1 (5,1,3,4,7,1, 15,12,13,12,12,12)
    Table 0593B8: (OK)order positions at checkp2 (12,5,5,7,18,5, 20,16,13,16,25,525)
    Table 059F02: (OK) screen with 1st door (19,19,19,15,19,14, -1,-1,-1,-1)
    Table 059EF7: (OK) screen with 2nd door (22,22,-1,18,-1,17,-1,-1,-1,34,32)
                  usually boss room-1, but not always there's a shutter
    
    Table CE31: Tile that replaces shutter at second door (if zero, use same as 1st)
    Table CE3D: Tile that replaces shutter at first door
    Table CE49: Screen where second shutter is (always the last room of scene)
    Table DAE6: Lift data (for Gutsman and Wily4)
    Table 05A888: Gutsman debris graphics tile
    
    Table ED9A: (OK)Boss initial X coord ;usually C0; for Wily2 it's B0.
    Table EDA3: (OK)Boss initial Y coord ;usually B4, for Gutsman it's B0.
                 ; Boss coordinates are not listed for Wily4. Wily3 is 00,00.

    ; Note: for Wily2 rematches, X coord is at 1DC13 lda #$40
    
    Table C51A: (OK) Teleport Y coord at start    ;usually 94,B4,C4,64,54...
    Table C526: (OK) Teleport Y coord at restartA
    Table C532: (OK) Teleport Y coord at restartB
    
    These two instructions determine Megaman's starting position
    when he teleports into the stage (no matter what situation)
        $910C> lda #$80 ;x
        $9113> lda #$04 ;y
    These two instructions determine Megaman's position when he
    enters a teleport in Wily4
        $9361> lda #$10 ;x
        $9366> lda #$B4 ;y
    This instruction determines Megaman's position after horizontal
    scrolling (it's responsible for the left-scrolling bug):
        $A054> lda #$18
    
    Table D06E: Index into 8F80 for "open first door"
    Table D07C: Index into 8F80 for "open second door"
    Table D087: Index into 8F80 for "close boss door"
    
    8F00..8F3F appears to be unused (except for a copyright message in bank 4)
    
SHUTTERS
    Table 8F80... 
      ; byte 0: -> length of the shutter (4 is typical)
      ; byte 1+n*2: -> address where to write (+4 increments Y by 2, +32 increments X by 2)
      ; byte 2+n*2: -> block number, which are loaded from 8F40 and 8F70
    Table 8F40:
      contains fourbyte blocks.
    Table 8F70:
      contains the palettes for each fourbyte block.

BUZZBLOCKS
   Most of this stuff is hardcoded into the assembly code!!
    Code address 05A11A: Compares the stage number if it's Ice or Elec.
    Code address 05A12B: Sets the ActiveList pointer for Iceman stage.
    code address 05A12F: Sets the ActiveList pointer for Elecman stage.
    Code address 05B40E: Tests if it's Iceman stage. 
    Code address 05B414: Tests if it's room number 10 (Iceman)
    Code address 05B418: Tests if it's other room (Iceman)
    Code address 05B420: Tests if it's room number 5 (Elecman)
    Address 05B486: 4 pointers, addresses of block-control data for each stage
    (List 0: iceman stage room 10,
     List 1: iceman stage room 2
     List 2: other stage room 5
     List 3: other stage other room)
    Address 05B48E..0584C1: The block data. The data describes the index
     into the active data that describes where the block is, and the wait
     until the next block is displayed.

TRANSPORTER LIFTS
   Most of this stuff is hardcoded into the assembly code!!
    Code address $DA86: Compares the stage number if it's Guts or Wily4.
    Code address $DA8C: Specifies 3 lifts for Gutsman stage.
    Code address $DA92: Specifies 1 lift for Wily4 stage.
    The lift data is at $DAE6.

*/

enum
{
    Up      = 2,
    Down    = 4,
    Shutter = 8,
    LevelBegin    = 16,
    RestartPointA = 32,
    RestartPointB = 64,
    Right   = 1,
    End     = 1,
    NoRooms = 0
};

// Routine $C39C: load the page of the level

#ifdef OBJECTS_CC
# define Vdef
# define Vval(v) = v
#else
# define Vdef extern
# define Vval(v) 
#endif

/* THESE ADDRESSES ARE RELATIVE TO LEVEL START PAGE.
 * THEREFORE, DON'T USE $8000..$BFFF, BUT USE $0000..$3FFF.
 */

// In each page, these settings are constant:
Vdef unsigned Addr_Blocks     Vval(0x0000); // Max blocks: 192 (shared)
Vdef unsigned Addr_BlockPals  Vval(0x0300); // Max blocks: 192 (shared)
Vdef unsigned Addr_Rooms      Vval(0x03C0); // Max rooms: 33   (shared)
/* "Rooms": Storage for rooms. Each rooom takes 64 bytes.
 * "Addr_Pointers" will point here.
 */

Vdef unsigned Addr_Screens    Vval(0x0C00); // Max screens: 48 (shared)
/* "Screens": For each 48 different screens, contains an index to "Pointers".
 * It converts a "screen num" into a "room num".
 * Aka. RoomOrderTable
 */

Vdef unsigned Addr_Pointers   Vval(0x0C30); // Max rooms: 32.  (shared)
/* "Pointers": Points to "Rooms". Indexed by an "screen".
 *
 * It converts a "room num" into a "room data pointer".
 */

Vdef unsigned Addr_Layout   Vval(0x0C70); // Max screens: 48. (shared)
/* "Layout": It tells how the stage scrolls.
 */

Vdef unsigned Addr_Palette1 Vval(0x0CA0);
Vdef unsigned Addr_Palette2 Vval(0x0CD0);

// Map1 ?table (number, followed by 2 bytes each)
// First byte:  A to SwitchChr (2 low bits is page, 6 high is block number)
// Second byte: the number of 256-byte entities to copy.
// This table is consumed entirely by a single call of WriteChr.
// This table seems to define which graphics the level
// is constructed from.
Vdef unsigned Addr_Tiles1   Vval(0x0D00);
// Same as Tiles1, but for the Wily version
Vdef unsigned Addr_Tiles2   Vval(0x0D20);

// Something common to both. Unit: 1 byte. Indexed by screen id.
Vdef unsigned Addr_MonsterGraphicsIndex   Vval(0x0D40);

// Something common to both. Unit: 2 bytes
// In 0D80-table, either one of the bytes 
// is a (block index*4)+(page number).
//
// One 32-byte entity will be copied from the block data into $382.

// Indexed by (value from D40) PLUS something*something
//
// Seems to contain 11-byte records,
// where 5 bytes are object numbers (or similar) and 6 bytes are palette.

Vdef unsigned Addr_MonsterGraphicsTable   Vval(0x0D80);

Vdef unsigned Addr_Actives1   Vval(0x0E00);
Vdef unsigned Addr_Actives2   Vval(0x0EA0);



// Bank 5: (reset bank):
//   8EC0 .. 8EC7 -> $2007  eight times
//   8EC0 .. 8ECF -> $2006 (lo)
//   8ED0 .. 8EDF -> $2006 (hi)
//   8EE0 .. 8EEF -> $2007

Vdef unsigned Addr_ActivesEnd Vval(0x0F00);

/* Address range 0F00..0F3F is unused, except
 * for licensed-blah-blah message in page 4.
 */
/* Addresses 0F40..0FFF are for shutter definitions. */

Vdef unsigned Addr_GFXstart   Vval(0x1000);

  // E00: 1 byte: number of objects (map1) Including terminator
  //      6 bytes per object. Terminator = 2nd byte is $FF
  //      Max objects: 29
  //  1: type
  //       2=gutsblock, 3=door_right, 1=door_left, 0=appearing block
  //       4=disintegrated gutsblock
  //  2: screen #
  //  3: x1 (in)
  //  4: y1 (in)
  //  5: x2 (out)
  //  6: y2 (out)
  // EA0: 1 byte: number of objects (map2) Including terminator
  //      6 bytes per object. Terminator = 2nd byte is $FF
  //      Max objects: 13
  // F00: ?
  //1000: gfx tiles actually begin here.

const unsigned Addr_EnemyPointers = 0x06A452; //C,I,B,F,E,G,1,2,3,4,? point to A468+.
static /*changed by encode.cc */
/***/ unsigned Addr_EnemyData     = 0x06A468; // room for 419 enemies (total).
const unsigned Addr_EnemyEnd      = 0x06AB00;

/* When you die after this screen, you will be returned to the restart point */
const unsigned Addr_CheckPointsScreenTable1 = 0xC2D4;
const unsigned Addr_CheckPointsScreenTable2 = 0xC2D4+12;
/* The restart point has this particular screen number */
const unsigned Addr_FirstScreenTable = 0xC2EB;
const unsigned Addr_RestartPointsScreenTable1 = 0xC2EB+12;
const unsigned Addr_RestartPointsScreenTable2 = 0xC2EB+24;
const unsigned Addr_RestartPointsScreenRepeatTable1 = 0x0593FA-12+12;
const unsigned Addr_RestartPointsScreenRepeatTable2 = 0x0593FA-12+24;
/* And this particular order number */
const unsigned Addr_FirstOrderTable  = 0x0593A0;
const unsigned Addr_RestartPointsOrderTable1 = 0x0593A0+12;
const unsigned Addr_RestartPointsOrderTable2 = 0x0593A0+24;
/* The restart point belongs to a strip that ends in this particular screen number */
const unsigned Addr_RestartPointsEndScreenTable1 = 0x059412-12+12;
const unsigned Addr_RestartPointsEndScreenTable2 = 0x059412-12+24;
/* And enemies are loaded up to this index */
const unsigned Addr_FirstEnemyTable  = 0x0593C3;
const unsigned Addr_RestartEnemyPointsTable1 = 0x0593C3+12;
const unsigned Addr_RestartEnemyPointsTable2 = 0x0593C3+24;
/* And teleport Y coordinates are loaded from here */
const unsigned Addr_TeleportYcoordTable = 0xC51A;
const unsigned Addr_RestartTeleportYcoordTable1 = 0xC51A+12;
const unsigned Addr_RestartTeleportYcoordTable2 = 0xC51A+24;
/* Boss coordinates are here */
const unsigned Addr_BossXcoord = 0xED9A;
const unsigned Addr_BossYcoord = 0xEDA3;

/* Screens where doors are */
const unsigned Addr_Door1ScreenTable = 0x059F02;
const unsigned Addr_Door2ScreenTable = 0x059EF7;

/* Address of the 10-byte table which lists the room numbers
 * of each level where the boss appears.
 */
const unsigned Addr_BossOffsetTable = 0xCE49;

const unsigned Addr_BossArrivalInputTable = 0x059380;
const unsigned Addr_AfterDoorPalette = 0x059F0D;

  // Enemies is list of 4-byte records, terminated by byte $FF.
  //  0: screen #
  //  1: x (center)
  //  2: y (center)
  //  3: type


// A level may contain 48 rooms, but when two levels are paired, the total amount must <= 48.
const unsigned MaxRooms   = 48;
// There is room for 13 actives at max per level.
const unsigned MaxActives = 32;
// There is room for 419 enemies at max in total.
const unsigned MaxEnemies = 256;
// There is room for 15 level gfx settings at max per level.
const unsigned MaxLevelGfx = 15;
// There is room for 11 monster gfx patterns at max per pair.
const unsigned MaxMonsterGfx = 11;

#endif
